@@ -84,10 +84,6 @@ class TimeField(Field, forms.TimeField):
     widget = TimeInput
 
 
-class DecimalField(Field, forms.DecimalField):
-    widget = NumberInput
-
-
 class FloatField(Field, forms.FloatField):
     widget = NumberInput
 
@@ -95,6 +91,10 @@ class FloatField(Field, forms.FloatField):
 class IntegerField(Field, forms.IntegerField):
     widget = NumberInput
 
+    def __init__(self, *args, **kwargs):
+        kwargs.setdefault('widget', NumberInput if not kwargs.get('localize') else self.widget)
+        super(IntegerField, self).__init__(*args, **kwargs)
+
     def widget_attrs(self, widget):
         attrs = super(IntegerField, self).widget_attrs(widget) or {}
         if self.min_value is not None:
@@ -104,6 +104,14 @@ class IntegerField(Field, forms.IntegerField):
         return attrs
 
 
+class DecimalField(Field, forms.DecimalField):
+    widget = NumberInput
+
+    def __init__(self, *args, **kwargs):
+        kwargs.setdefault('widget', NumberInput if not kwargs.get('localize') else self.widget)
+        super(DecimalField, self).__init__(*args, **kwargs)
+
+
 class EmailField(Field, forms.EmailField):
     widget = EmailInput
 
