@@ -23,6 +23,7 @@ class ConfigFilter(object):
     * the bound field passed into the constructor equals the filtered field.
     * the string passed into the constructor equals the fields name.
     * the string passed into the constructor equals the field's class name.
+    * the string passed into the constructor equals the field's widget class name.
     """
     def __init__(self, var):
         self.var = var
@@ -37,7 +38,17 @@ class ConfigFilter(object):
                     return True
         if self.var == bound_field.name:
             return True
+        # ignore 'object' in the mro, because it would be a match-all filter
+        # anyway. And 'object' could clash with a field that is named the
+        # same.
         for class_ in bound_field.field.__class__.__mro__:
+            if class_.__name__ == 'object':
+                continue
+            if self.var == class_.__name__:
+                return True
+        for class_ in bound_field.field.widget.__class__.__mro__:
+            if class_.__name__ == 'object':
+                continue
             if self.var == class_.__name__:
                 return True
 
@@ -299,12 +310,12 @@ class ModifierBase(BaseFormNode):
         filter = None
         if self.options['for']:
             try:
-                var = self.options['for'].resolve(context)
+                for_ = self.options['for'].resolve(context)
             except VariableDoesNotExist:
                 if settings.TEMPLATE_DEBUG:
                     raise
                 return ''
-            filter = ConfigFilter(var)
+            filter = ConfigFilter(for_)
         if self.options['using']:
             try:
                 template_name = self.options['using'].resolve(context)
@@ -449,6 +460,9 @@ class BaseFormRenderNode(BaseFormNode):
     def render(self, context):
         only = self.options['only']
 
+        config = self.get_config(context)
+        config.push()
+
         extra_context = self.get_extra_context(context)
         nodelist = self.get_nodelist(context, extra_context)
         if nodelist is None:
@@ -456,12 +470,14 @@ class BaseFormRenderNode(BaseFormNode):
 
         if only:
             context = context.new(extra_context)
-            return nodelist.render(context)
+            output = nodelist.render(context)
         else:
             context.update(extra_context)
             output = nodelist.render(context)
             context.pop()
-            return output
+
+        config.pop()
+        return output
 
 
 class FormNode(BaseFormRenderNode):
@@ -611,12 +627,16 @@ class FormFieldNode(BaseFormRenderNode):
             context.update(extra_context)
             context_instance = context
 
+        config.push()
+
         # Using a context manager here until Django's BoundField takes
         # template name and context instance parameters
         with attributes(widget, template_name=template_name,
                         context_instance=context_instance) as widget:
             output = bound_field.as_widget(widget=widget)
 
+        config.pop()
+
         if not self.options['only']:
             context.pop()
 
