@@ -1,3 +1,6 @@
+import warnings
+warnings.simplefilter('always')
+
 DATABASES = {
     'default': {
         'ENGINE': 'django.db.backends.sqlite3',
@@ -5,6 +8,9 @@ DATABASES = {
     },
 }
 
+USE_I18N = True
+USE_L10N = True
+
 INSTALLED_APPS = [
     'django.contrib.gis',
     'floppyforms',
@@ -13,6 +19,8 @@ INSTALLED_APPS = [
 
 STATIC_URL = '/static/'
 
-TEST_RUNNER = 'discover_runner.DiscoverRunner'
-
 SECRET_KEY = '0'
+
+import django
+if django.VERSION < (1, 6):
+    TEST_RUNNER = 'discover_runner.DiscoverRunner'
