@@ -1,3 +1,4 @@
+import django
 from django.forms import TextInput
 from django.forms.formsets import formset_factory
 from django.template import Context, Template, TemplateSyntaxError
@@ -9,6 +10,12 @@ from floppyforms.templatetags.floppyforms import (FormConfig, ConfigFilter,
                                                   FieldModifier)
 
 
+_TEMPLATE_PREAMBLE = '{% load floppyforms %}'
+if django.VERSION >= (1, 6):
+    _TEMPLATE_PREAMBLE += '{% load firstof from future %}'
+    _TEMPLATE_PREAMBLE += '{% load cycle from future %}'
+
+
 def render(template, context=None, config=None):
     if context is None:
         context = {}
@@ -16,7 +23,7 @@ def render(template, context=None, config=None):
         context = Context(context)
     if config is not None:
         setattr(context, FormNode.CONFIG_CONTEXT_ATTR, config)
-    t = Template('{% load floppyforms %}' + template)
+    t = Template(_TEMPLATE_PREAMBLE + template)
     return t.render(context)
 
 
@@ -40,6 +47,7 @@ class PersonForm(forms.Form):
     firstname = forms.CharField()
     lastname = forms.CharField()
     age = forms.IntegerField()
+    bio = forms.CharField(widget=forms.Textarea)
 
 
 class HardcodedWidget(forms.Widget):
@@ -189,6 +197,9 @@ class FormConfigNodeTests(TestCase):
         self.assertEqual(
             config.retrieve('widget_template', bound_field=form['age']),
             'floppyforms/input.html')
+        self.assertEqual(
+            config.retrieve('widget_template', bound_field=form['bio']),
+            'floppyforms/textarea.html')
 
     def test_field_config_for_field_name(self):
         form = PersonForm()
@@ -206,6 +217,9 @@ class FormConfigNodeTests(TestCase):
         self.assertEqual(
             config.retrieve('widget_template', bound_field=form['age']),
             'floppyforms/input.html')
+        self.assertEqual(
+            config.retrieve('widget_template', bound_field=form['bio']),
+            'floppyforms/textarea.html')
 
     def test_field_config_for_field_type(self):
         form = PersonForm()
@@ -223,6 +237,29 @@ class FormConfigNodeTests(TestCase):
         self.assertEqual(
             config.retrieve('widget_template', bound_field=form['age']),
             'field.html')
+        self.assertEqual(
+            config.retrieve('widget_template', bound_field=form['bio']),
+            'floppyforms/textarea.html')
+
+    def test_field_config_for_widget_type(self):
+        form = PersonForm()
+        context = Context({FormNode.IN_FORM_CONTEXT_VAR: True, 'form': form})
+
+        node = compile_to_nodelist('{% formconfig field using "field.html" for "Textarea" %}')
+        node.render(context)
+        config = node.get_config(context)
+        self.assertEqual(
+            config.retrieve('widget_template', bound_field=form['firstname']),
+            'floppyforms/input.html')
+        self.assertEqual(
+            config.retrieve('widget_template', bound_field=form['lastname']),
+            'floppyforms/input.html')
+        self.assertEqual(
+            config.retrieve('widget_template', bound_field=form['age']),
+            'floppyforms/input.html')
+        self.assertEqual(
+            config.retrieve('widget_template', bound_field=form['bio']),
+            'field.html')
 
 
 class FormTagTests(TestCase):
@@ -299,7 +336,7 @@ class FormTagTests(TestCase):
                 {% endif %}
                 Length: {{ forms|length }}
             {% endform %}""", {
-                'f1': SimpleForm(),
+                'f1': SimpleForm(),  # noqa
                 'f2': SimpleForm()
             }), 'Equals! Length: 2')
 
@@ -310,7 +347,7 @@ class FormTagTests(TestCase):
                     'myform': PersonForm(),
                 }), """
                 Forms: 1
-                1. Form Fields: firstname lastname age
+                1. Form Fields: firstname lastname age bio
                 """)
         with self.assertTemplateUsed('simple_form_tag.html'):
             self.assertHTMLEqual(
@@ -320,7 +357,7 @@ class FormTagTests(TestCase):
                 }), """
                 Forms: 2
                 1. Form Fields: name
-                2. Form Fields: firstname lastname age
+                2. Form Fields: firstname lastname age bio
                 """)
 
     def test_include_content_with_extra_arguments(self):
@@ -330,7 +367,7 @@ class FormTagTests(TestCase):
                     'myform': PersonForm(),
                 }), """
                 Forms: 1
-                1. Form Fields: firstname lastname age
+                1. Form Fields: firstname lastname age bio
                 Extra argument: spam
                 """)
         with self.assertTemplateUsed('simple_form_tag.html'):
@@ -339,7 +376,7 @@ class FormTagTests(TestCase):
                     'myform': PersonForm(),
                 }), """
                 Forms: 1
-                1. Form Fields: firstname lastname age
+                1. Form Fields: firstname lastname age bio
                 """)
         with self.assertTemplateUsed('simple_form_tag.html'):
             self.assertHTMLEqual(
@@ -350,7 +387,7 @@ class FormTagTests(TestCase):
                     """, {'myform': PersonForm()}),
                 """
                 Forms: 1
-                1. Form Fields: firstname lastname age
+                1. Form Fields: firstname lastname age bio
                 Extra argument: ham
                 """)
         with self.assertTemplateUsed('simple_form_tag.html'):
@@ -362,7 +399,7 @@ class FormTagTests(TestCase):
                     """, {'myform': PersonForm()}),
                 """
                 Forms: 1
-                1. Form Fields: firstname lastname age
+                1. Form Fields: firstname lastname age bio
                 """)
         with self.assertTemplateUsed('simple_form_tag.html'):
             self.assertHTMLEqual(
@@ -373,7 +410,7 @@ class FormTagTests(TestCase):
                     """, {'myform': PersonForm()}),
                 """
                 Forms: 1
-                1. Form Fields: firstname lastname age
+                1. Form Fields: firstname lastname age bio
                 """)
 
     def test_default_template(self):
@@ -389,7 +426,7 @@ class FormTagTests(TestCase):
                 'forms': form_list,
             }), """
             Forms: 3
-            1. Form Fields: firstname lastname age
+            1. Form Fields: firstname lastname age bio
             2. Form Fields: name
             3. Form Fields: name
             """)
@@ -402,9 +439,9 @@ class FormTagTests(TestCase):
                 'formset': formset,
             }), """
             Forms: 3
-            1. Form Fields: firstname lastname age
-            2. Form Fields: firstname lastname age
-            3. Form Fields: firstname lastname age
+            1. Form Fields: firstname lastname age bio
+            2. Form Fields: firstname lastname age bio
+            3. Form Fields: firstname lastname age bio
             """)
 
         formset = PersonFormSet(initial=[{}, {}])
@@ -413,13 +450,36 @@ class FormTagTests(TestCase):
                 'formset': formset,
             }), """
             Forms: 5
-            1. Form Fields: firstname lastname age
-            2. Form Fields: firstname lastname age
-            3. Form Fields: firstname lastname age
-            4. Form Fields: firstname lastname age
-            5. Form Fields: firstname lastname age
+            1. Form Fields: firstname lastname age bio
+            2. Form Fields: firstname lastname age bio
+            3. Form Fields: firstname lastname age bio
+            4. Form Fields: firstname lastname age bio
+            5. Form Fields: firstname lastname age bio
             """)
 
+    def test_formconfig_gets_popped_after_form_tag(self):
+        form = PersonForm()
+        rendered = render('''{% form form using %}
+            {% formconfig row with extra_argument="first argument" %}
+            {% formrow form.firstname using "simple_formrow_tag.html" %}
+            {% form form using %}
+                {% formconfig row with extra_argument="pop me" %}
+                {% formrow form.lastname using "simple_formrow_tag.html" %}
+                {% formrow argument|if:forloop.last|else:None %}
+            {% endform %}
+            {% formrow form.lastname using "simple_formrow_tag.html" %}
+        {% endform %}''', {'form': form})
+        self.assertHTMLEqual(rendered, '''
+        Fields: 1
+        1. Field: firstname Extra argument: first argument
+
+        Fields: 1
+        1. Field: lastname Extra argument: pop me
+
+        Fields: 1
+        1. Field: lastname Extra argument: first argument
+        ''')
+
 
 class FormRowTagTests(TestCase):
     def test_valid_syntax(self):
@@ -559,24 +619,47 @@ class FormRowTagTests(TestCase):
         self.assertHTMLEqual(render("""{% form myform using %}
             {% formrow form using "simple_formrow_tag.html" %}
         {% endform %}""", {'myform': form}), """
-            Fields: 3
+            Fields: 4
             1. Field: firstname
             2. Field: lastname
             3. Field: age
+            4. Field: bio
         """)
 
         form = PersonForm()
         self.assertHTMLEqual(render("""{% form myform using %}
             {% formrow form.lastname form None form.firstname using "simple_formrow_tag.html" %}
         {% endform %}""", {'myform': form}), """
-            Fields: 5
+            Fields: 6
             1. Field: lastname
             2. Field: firstname
             3. Field: lastname
             4. Field: age
-            5. Field: firstname
+            5. Field: bio
+            6. Field: firstname
         """)
 
+    def test_formconfig_gets_popped_after_formrow_tag(self):
+        '''
+        Tests that the form config will be reseted after being set in a
+        ``formrow`` tag.
+        '''
+        form = SimpleForm()
+        rendered = render('''{% form form using %}
+            {% formconfig row with extra_argument="first argument" %}
+            {% formrow form.name using "simple_formrow_tag_with_config.html" %}
+            {% formrow form.name using "simple_formrow_tag.html" %}
+        {% endform %}''', {'form': form})
+        self.assertHTMLEqual(rendered, '''
+        Fields: 1
+        1. Field: name argument: defined inline
+        Extra argument: first argument
+
+        Fields: 1
+        1. Field: name
+        Extra argument: first argument
+        ''')
+
 
 class FormFieldTagTests(TestCase):
     def test_valid_syntax(self):
@@ -654,6 +737,22 @@ class FormFieldTagTests(TestCase):
             {% formfield form.name %}
         {% endform %}""", {'myform': form}), """Hardcoded widget.""")
 
+    def test_formconfig_gets_popped_after_formfield_tag(self):
+        '''
+        Tests that the form config will be reseted after being set in a
+        ``formfield`` tag.
+        '''
+        form = SimpleForm()
+        rendered = render('''{% form form using %}
+            {% formconfig field with extra_argument="first argument" %}
+            {% formfield form.name using "extra_argument_with_config.html" with prefix="1." %}
+            {% formfield form.name using "extra_argument.html" with prefix="2." %}
+        {% endform %}''', {'form': form})
+        self.assertHTMLEqual(rendered, '''
+        1. argument: first argument
+        2. argument: first argument
+        ''')
+
 
 class WidgetTagTest(TestCase):
     def test_widget_tag(self):
